/* lmpvartable.c
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include "lmp.h"
#include "lmpinput.h"
#include "lmpmath.h"

/* var table */
var_table_t * lmp_vt = NULL;
/* var table size */
int lmp_vt_size = 0;

/* search a variable in the var table
 * returns 0 if successful, else 1
 */
int lmp_vt_lookup (s, val)
const char * s;       /* to be compared string */
lmp_value_type * val; /* return value */
{
  int i;

  for (i = 0; i < lmp_vt_size; i++)
    if (!strcmp (s, lmp_vt [i].var)) {
      *val = lmp_vt [i].val;
      return 0;
    }

  return 1;
}

int lmp_vt_add (var,val)
const char * var;
lmp_value_type val;
{
  char * endptr;
  lmp_value_type test;

  test = STRTOFVAL (var, &endptr);
  if (*endptr != 0 && test == 0) {
    int i;

    for (i = 0; i < lmp_vt_size; i++) {
      if (!strcmp (lmp_vt [i].var, var)) {
	lmp_vt [i].val = val;
	return 0;
      }
    }
    lmp_vt_size++;
    lmp_vt = realloc (lmp_vt, lmp_vt_size * sizeof (var_table_t));
    lmp_vt [lmp_vt_size-1].var = malloc (strlen (var) + 1);
    strcpy (lmp_vt [lmp_vt_size-1].var, var);
    lmp_vt [lmp_vt_size-1].val = val;
  }
  else {
    lmp_error ("Very funny: redefining numbers ;-)\n");
  }

  return 0;
}

/* initialize the var table
 * returns 0 if successful
 */
int lmp_vt_init (vs)
const char * vs; /* initializer string: "x=3:y=-0.5..." */
{
  char * sp, * tok, * var, * val, * endptr;

  if (lmp_vt)
    free (lmp_vt);

  lmp_vt_size = 0;
  

  sp = malloc (strlen (vs) + 1);
  strcpy (sp, vs);

  while ((sp) && ((tok = GETTOKEN (sp, ":")) != NULL)) {
    if (tok)
      var = GETTOKEN (tok, "=");
    else
      lmp_error ("Init string invalid, perhaps a trailing ':'.\n");

    if (tok)
      val = GETTOKEN (tok, "=");
    else
      lmp_error ("Init string invalid, perhaps a missing '=' or trailing ':'.\n");

    if (val[0] == '{') { // is complex: this is a hack
#ifdef HAVE_COMPLEX
      lmp_value_type temp;

      val [0] = ' ';                 // remove leading '}'
      val [strlen (val) - 1] = '\0'; // remove closing '}'
      temp = STRTOFVAL (GETTOKEN (val, ","), &endptr);
      if (val == NULL)
	lmp_error ("Init string syntax error.\n");
      if (*endptr != 0 && temp == 0)
	lmp_error ("Init string error: %s has wrong real part.\n", var);
      temp += STRTOFVAL (val, &endptr) * _Complex_I;
      if (*endptr != 0 && CIMAG (temp) == 0)
	lmp_error ("Init string error: %s has wrong imaginary part.\n", var);
      lmp_vt_add (var, temp);
#else
      lmp_error ("This binary does not support complex values.\n");
#endif
    }
    else
      lmp_vt_add (var, STRTOFVAL (val, &endptr));

    if (*endptr != 0 && lmp_vt [lmp_vt_size-1].val == 0)
	lmp_error ("Init string error: %s is no number\n", val);
  }

  if (*endptr != 0 && lmp_vt [lmp_vt_size-1].val == 0)
      lmp_error ("Init string error: %s is no number.\n", val);
  free (sp);

  return 0;
}

void lmp_vt_dump () {
  int i;

  for (i = 0; i < lmp_vt_size; i++) {
    printf ("%d: %s => {%" FREAL ",%" FREAL "}\n", i, lmp_vt [i].var, CREAL (lmp_vt [i].val), CIMAG (lmp_vt [i].val));
  }
}
